package com.qs.mp.channel.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qs.mp.channel.domain.Channel;
import com.qs.mp.channel.domain.ChannelAddr;
import com.qs.mp.channel.domain.ChannelCommission;
import com.qs.mp.channel.domain.vo.ChannelOperDataVO;
import com.qs.mp.channel.domain.vo.ChannelVO;
import com.qs.mp.channel.mapper.ChannelMapper;
import com.qs.mp.channel.service.IChannelCommissionService;
import com.qs.mp.channel.service.IChannelService;
import com.qs.mp.channel.service.IChannelUserRelService;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.enums.ErrorCodeEnum;
import com.qs.mp.common.exception.ServiceException;
import com.qs.mp.system.domain.SysUser;
import com.qs.mp.system.service.ISysUserService;
import com.qs.mp.system.service.id.BizIdGenerator;
import com.qs.mp.utils.SecurityUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 渠道经销商 服务实现类
 * </p>
 *
 * @author quanshu
 * @since 2022-03-02
 */
@Service
public class ChannelServiceImpl extends ServiceImpl<ChannelMapper, Channel> implements IChannelService {

	
	@Autowired
	private IChannelService channelService;
	
	@Autowired
	private IChannelUserRelService channelUserRelService;
	
	@Autowired
	private IChannelCommissionService channelCommissionService;
	
    @Autowired
    private ISysUserService userService;

	@Autowired
	private BizIdGenerator bizIdGenerator;

	@Override
	@Transactional
	public void saveChannel(Channel channel) throws Exception {
		// 1、校验名称是否重复、手机号是否存在（渠道表）
		int nameCount = channelService.count(
		        new LambdaQueryWrapper<Channel>().eq(Channel::getName, channel.getName()).gt(Channel::getLevel, 1));
		if(nameCount > 0) {
			 throw new ServiceException("渠道名称" + channel.getName() + "已存在！");
		}
		int mobileCount = channelService.count(
		        new LambdaQueryWrapper<Channel>().eq(Channel::getMobile, channel.getMobile()));
		if(mobileCount > 0) {
			 throw new ServiceException("手机号码" + channel.getMobile() + "已注册！");
		}
		// 2.校验佣金比例，不能高于其父渠道的佣金比例
		Channel parentChannel = channelService.getById(channel.getParentId());
		if(null != parentChannel && null != parentChannel.getCommRate()) {
			 if(channel.getCommRate().compareTo(parentChannel.getCommRate()) > 0) {
				 throw new ServiceException("渠道佣金比例不能高于父渠道的佣金比例");
			 }
		}
		channel.setLevel(parentChannel.getLevel()+1);
		// 3.插入数据
		boolean res = channelService.save(channel);
		if(res && null != channel.getChannelId()) {
			channel.setChannelNo(parentChannel.getChannelNo()+"."+channel.getChannelId());
			// 生成用户
			SysUser sysUser = new SysUser();
			sysUser.setUserName(channel.getMobile());
		    sysUser.setPhonenumber(channel.getMobile());
		    sysUser.setNickName(channel.getName());
		    sysUser.setDeptId(SecurityUtils.getDeptId());
			sysUser.setCreateBy(SecurityUtils.getUsername());
			int userNum = userService.insertUser(sysUser);
		    if (userNum == 0 || null == sysUser.getUserId()) {
		        throw new ServiceException("新增渠道'" + channel.getName() + "'失败，请联系管理员");
		    }
		    channel.setUserId(sysUser.getUserId());
		    channelService.updateById(channel);
		    // 4.发送账号创建成功短信
		}
	}
	
	
	@Override
	@Transactional
	public void saveSite(Channel channel) throws Exception {
		// 1、校验名称是否重复、手机号是否存在（渠道表）
		int nameCount = channelService.count(
		        new LambdaQueryWrapper<Channel>().eq(Channel::getName, channel.getName()).eq(Channel::getLevel, 0));
		if(nameCount > 0) {
			 throw new ServiceException("经销商名称" + channel.getName() + "已存在！");
		}
		int mobileCount = channelService.count(
		        new LambdaQueryWrapper<Channel>().eq(Channel::getMobile, channel.getMobile()));
		if(mobileCount > 0) {
			 throw new ServiceException("手机号码" + channel.getMobile() + "已注册！");
		}
		// 2.校验佣金比例，不能高于其父渠道的佣金比例
		Channel parentChannel = channelService.getById(channel.getParentId());
		if(null != parentChannel && null != parentChannel.getCommRate()) {
			 if(channel.getCommRate().compareTo(parentChannel.getCommRate()) > 0) {
				 throw new ServiceException("经销商佣金比例不能高于父渠道的佣金比例");
			 }
		}
		channel.setLevel(0);
		// 3.插入数据
		boolean res = channelService.save(channel);
		if(res && null != channel.getChannelId()) {
			channel.setChannelNo(parentChannel.getChannelNo()+"."+channel.getChannelId());
			// 生成用户
			SysUser sysUser = new SysUser();
			sysUser.setUserName(channel.getMobile());
		    sysUser.setPhonenumber(channel.getMobile());
		    sysUser.setNickName(channel.getName());
		    sysUser.setDeptId(SecurityUtils.getDeptId());
			sysUser.setCreateBy(SecurityUtils.getUsername());
			int userNum = userService.insertUser(sysUser);
		    if (userNum == 0 || null == sysUser.getUserId()) {
		        throw new ServiceException("新增经销商'" + channel.getName() + "'失败，请联系管理员");
		    }
		    channel.setUserId(sysUser.getUserId());
		    channelService.updateById(channel);
		    // 4.发送账号创建成功短信
		}
	}
	

	@Override
	public void updateChannel(Channel channel) throws Exception {
		// 1、校验修改子渠道是否为当前用户的子渠道
		Long channelId = SecurityUtils.getLoginUser().getChannelId();
		
		// 2.校验名称是否重复、手机号是否存在（渠道表）；
		
		// 3.校验佣金比例，不能高于其父渠道的佣金比例，不能低于其子渠道的最大佣金比例
		
	    // 4.更新入库
	}
	
	@Override
	public void updateSite(Channel channel) throws Exception {
		// 1、校验修改子渠道是否为当前用户的子渠道
		Long channelId = SecurityUtils.getLoginUser().getChannelId();
		
		// 2.校验名称是否重复、手机号是否存在（渠道表）；
		
		// 3.校验佣金比例，不能高于其父渠道的佣金比例，不能低于其子渠道的最大佣金比例
		
	    // 4.更新入库
	}

	@Override
	public List<ChannelVO> selectChannelVoList(Channel channel) {
		QueryWrapper<Channel> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("t1.parent_id", channel.getParentId());
		queryWrapper.gt("t1.level", 0);
		return getBaseMapper().selectChannelVoList(queryWrapper);
	}
	
	@Override
	public List<ChannelVO> selectSiteVoList(Channel channel) {
		QueryWrapper<Channel> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("t1.parent_id", channel.getParentId());
		queryWrapper.eq("t1.level", 0);
		if(null != channel.getVerifyStatus()) {
			queryWrapper.eq("t1.verify_status", channel.getVerifyStatus());
		}
		if(null != channel.getCertifyStatus()) {
			queryWrapper.eq("t1.certify_status", channel.getCertifyStatus());
		}
		return getBaseMapper().selectSiteVoList(queryWrapper);
	}

	@Override
	public Channel getChannelByUserId(Long userId) {
		return getBaseMapper().selectOne(new LambdaQueryWrapper<Channel>().eq(Channel::getUserId, userId));
	}


	@Override
	public ChannelOperDataVO getChannelOperData(String channelNo, int days) {
		ChannelOperDataVO channelOperData = new ChannelOperDataVO();
		ChannelCommission channelCommission = channelCommissionService.getChannelCommissionCnt(channelNo, days);
		if(null != channelCommission) {
			channelOperData.setSaleAmt(channelCommission.getSaleAmt());
			channelOperData.setCommAmt(channelCommission.getCommAmt());
		}
		int newUserCnt = channelUserRelService.getChannelUserCnt(channelNo, days);
		channelOperData.setNewUserCnt(newUserCnt);
		return channelOperData;
	}
}
